//
//      contrast - compare files token by token
//      Copyright (C) 2006, 2009 Peter Miller
//
//      This program is free software; you can redistribute it and/or
//      modify it under the terms of the GNU General Public License,
//      version 3, as published by the Free Software Foundation.
//
//      This program is distributed in the hope that it will be useful,
//      but WITHOUT ANY WARRANTY; without even the implied warranty of
//      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//      GNU General Public License for more details.
//
//      You should have received a copy of the GNU General Public License
//      along with this program. If not, see
//      <http://www.gnu.org/licenses/>.
//

#include <cstdio>
#include <cstdlib>
#include <unistd.h>

#include <lib/comparator.h>
#include <lib/print_version.h>
#include <lib/progname.h>
#include <lib/source.h>
#include <lib/token/list.h>


static void
usage()
{
    const char *prog = progname_get();
    fprintf(stderr, "Usage: %s [ <option>... ] -A <file1> <file2>\n", prog);
    fprintf(stderr, "       %s [ <option>... ] -T <filename>\n", prog);
    fprintf(stderr, "       %s -l\n", prog);
    fprintf(stderr, "       %s -L\n", prog);
    fprintf(stderr, "       %s -V\n", prog);
    exit(1);
}


static void
broken(int, char **)
{
    usage();
}


static rcstring tokenizer_name;


static void
debug_tokenizer(int ac, char **av)
{
    if (ac != 1)
        usage();
    token_list tokens;
    tokens.read(av[0], tokenizer_name);
    tokens.print();
}


static rcstring algorithm_name;


static void
debug_algorithm(int ac, char **av)
{
    if (ac != 2)
        usage();
    token_list lhs;
    lhs.read(av[0], tokenizer_name);
    token_list rhs;
    rhs.read(av[1], tokenizer_name);
    comparator *cmp = comparator::factory(lhs, rhs, algorithm_name);
    cmp->compare();
    cmp->print_chunks();
    delete cmp;
}


int
main(int argc, char **argv)
{
    void (*mode)(int ac, char **av) = broken;
    rcstring tokenizer_name;
    rcstring algorithm_name;
    progname_set(argv[0]);
    for (;;)
    {
        int c = getopt(argc, argv, "Aa:LlTt:V");
        if (c == EOF)
            break;
        switch (c)
        {
        case 'A':
            mode = debug_algorithm;
            break;

        case 'a':
            algorithm_name = optarg;
            break;

        case 'L':
            source::list();
            return 0;

        case 'l':
            comparator::list();
            return 0;

        case 'T':
            mode = debug_tokenizer;
            break;

        case 't':
            tokenizer_name = optarg;
            break;

        case 'V':
            print_version();
            return 0;

        default:
            usage();
        }
    }
    mode(argc - optind, argv + optind);
    return 0;
}
