//
//      contrast - compare files token by token
//      Copyright (C) 2006, 2009 Peter Miller
//
//      This program is free software; you can redistribute it and/or
//      modify it under the terms of the GNU General Public License,
//      version 3, as published by the Free Software Foundation.
//
//      This program is distributed in the hope that it will be useful,
//      but WITHOUT ANY WARRANTY; without even the implied warranty of
//      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//      GNU General Public License for more details.
//
//      You should have received a copy of the GNU General Public License
//      along with this program. If not, see
//      <http://www.gnu.org/licenses/>.
//

#include <cstdio>
#include <cstdlib>
#include <unistd.h>

#include <lib/progname.h>
#include <lib/print_version.h>
#include <lib/comparator.h>


static void
usage()
{
    const char *prog = progname_get();
    fprintf(stderr, "Usage: %s [ -<option>... ] <file1> <file2>\n", prog);
    fprintf(stderr, "       %s -V\n", prog);
    exit(1);
}


int
main(int argc, char **argv)
{
    progname_set(argv[0]);
    for (;;)
    {
        int c = getopt(argc, argv, "V");
        if (c == EOF)
            break;
        switch (c)
        {
        case 'V':
            print_version();
            return 0;

        default:
            usage();
            // NOTREACHED
        }
    }
    if (optind + 2 != argc)
        usage();

    token_list lhs;
    lhs.read(argv[optind]);
    token_list rhs;
    rhs.read(argv[optind + 1]);

    comparator *cmp = comparator::factory(lhs, rhs);
    cmp->compare();

    //
    // Now we print the right-hand file, using the white space from the
    // left-hand file.
    //
    const chunk_list &chunks = cmp->get_chunks();
    for (size_t j = 0; j < chunks.size(); ++j)
    {
        const chunk &c = chunks[j];
        if (c.get_left_width() == 0)
        {
            // Tokens come from right-hand side only,
            // so just print them
            size_t w = c.get_right_width();
            for (size_t k = 0; k < w; ++k)
            {
                rhs[c.get_rlo() + k].print();
            }
        }
        else if (c.get_right_width() == 0)
        {
            // Tokens come from left-hand side only,
            // so ignore them.
        }
        else
        {
            // Tokens present on both sides (and are the same on bith
            // sides) so print the left hand side.
            size_t w = c.get_left_width();
            for (size_t k = 0; k < w; ++k)
            {
                lhs[c.get_llo() + k].print();
            }
        }
    }

    return 0;
}
