//
//      contrast - compare files token by token
//      Copyright (C) 2006, 2009 Peter Miller
//
//      This program is free software; you can redistribute it and/or
//      modify it under the terms of the GNU General Public License,
//      version 3, as published by the Free Software Foundation.
//
//      This program is distributed in the hope that it will be useful,
//      but WITHOUT ANY WARRANTY; without even the implied warranty of
//      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//      GNU General Public License for more details.
//
//      You should have received a copy of the GNU General Public License
//      along with this program. If not, see
//      <http://www.gnu.org/licenses/>.
//

#ifndef CONTRAST_COMPARISON_WINDOW_H
#define CONTRAST_COMPARISON_WINDOW_H

#include <gtkmm.h>

class chunk_list; // forward
class token_list; // forward

/**
  * The comparison_window class is used to represent the state of the
  * two panel GUI.
  */
class comparison_window:
    public Gtk::Window
{
public:
    /**
      * The destructor.
      */
    virtual ~comparison_window();

    /**
      * The constructor.
      */
    comparison_window(const chunk_list &chunks, const token_list &lhs,
        const token_list &rhs);

private:
    void left_redraw();
    void right_redraw();
    void go_current();

    //Signal handlers:
    void on_button_quit();
    void go_back();
    void go_next();
    void go_first();
    void left_as_left();
    void left_as_right();
    void right_as_left();
    void right_as_right();

    const chunk_list &chunks;
    const token_list &lhs;
    const token_list &rhs;

    enum hand_t
    {
        left_hand,
        right_hand
    };

    //
    //  ,-vbox---------------------------------------------------.
    //  | ,-side_by_side------------------------------------. |
    //  | | ,-left_vbox-----------. ,-right_vbox----------. | |
    //  | | | ,-left_caption----. | | ,-right_caption---. | | |
    //  | | | |                 | | | |                 | | | |
    //  | | | `-----------------' | | `-----------------' | | |
    //  | | |                     | |                     | | |
    //  | | | ,-left_scroller---. | | ,-right_scroller--. | | |
    //  | | | |                ^| | | |                ^| | | |
    //  | | | | left_text      #| | | | right_text     #| | | |
    //  | | | |                #| | | |                #| | | |
    //  | | | |                v| | | |                v| | | |
    //  | | | `-----------------' | | `-----------------' | | |
    //  | | |                     | |                     | | |
    //  | | | ,-left_buttons----. | | ,-right_buttons---. | | |
    //  | | | |                 | | | |                 | | | |
    //  | | | `-----------------' | | `-----------------' | | |
    //  | | `---------------------' `---------------------' | |
    //  | `-------------------------------------------------' |
    //  |                                                     |
    //  | ,-bottom_buttons----------------------------------. |
    //  | |                                         (Quit)  | |
    //  | `-------------------------------------------------' |
    //  `-----------------------------------------------------'
    //

    Gtk::VBox vbox;
    Gtk::HBox side_by_side;

    Gtk::VBox left_vbox;
    Gtk::ScrolledWindow left_scroller;
    Gtk::TextView left_text;
    Gtk::HBox left_buttons;
    Gtk::Button format_left_as_left_button;
    Gtk::Button format_left_as_right_button;

    Gtk::VBox right_vbox;
    Gtk::ScrolledWindow right_scroller;
    Gtk::TextView right_text;
    Gtk::HBox right_buttons;
    Gtk::Button format_right_as_left_button;
    Gtk::Button format_right_as_right_button;

    Gtk::HBox bottom_buttons;
    Gtk::Button back_button;
    Gtk::Button next_button;
    Gtk::Button quit_button;

    Glib::RefPtr<Gtk::TextBuffer::Tag> left_red_background_tag;
    Glib::RefPtr<Gtk::TextBuffer::Tag> left_red2_background_tag;
    Glib::RefPtr<Gtk::TextBuffer::Tag> right_green_background_tag;
    Glib::RefPtr<Gtk::TextBuffer::Tag> right_green2_background_tag;

    hand_t format_left_as;
    hand_t format_right_as;

    size_t current_chunk;
};

#endif // CONTRAST_COMPARISON_WINDOW_H
