//
//      contrast - compare files token by token
//      Copyright (C) 2006, 2009 Peter Miller
//
//      This program is free software; you can redistribute it and/or
//      modify it under the terms of the GNU General Public License,
//      version 3, as published by the Free Software Foundation.
//
//      This program is distributed in the hope that it will be useful,
//      but WITHOUT ANY WARRANTY; without even the implied warranty of
//      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//      GNU General Public License for more details.
//
//      You should have received a copy of the GNU General Public License
//      along with this program. If not, see
//      <http://www.gnu.org/licenses/>.
//

#include <cstdio>
#include <lib/chunk.h>
#include <lib/rcstring.h>


chunk::~chunk()
{
}


chunk::chunk() :
    llo(0),
    lhi(0),
    rlo(0),
    rhi(0)
{
}


chunk::chunk(size_t a_llo, size_t a_lhi, size_t a_rlo, size_t a_rhi) :
    llo(a_llo),
    lhi(a_lhi),
    rlo(a_rlo),
    rhi(a_rhi)
{
}


chunk::chunk(const chunk &arg) :
    llo(arg.llo),
    lhi(arg.lhi),
    rlo(arg.rlo),
    rhi(arg.rhi)
{
}


chunk &
chunk::operator=(const chunk &arg)
{
    if (this != &arg)
    {
        llo = arg.llo;
        lhi = arg.lhi;
        rlo = arg.rlo;
        rhi = arg.rhi;
    }
    return *this;
}


bool
chunk::valid()
    const
{
    return (llo <= lhi && rlo <= rhi);
}


bool
chunk::operator==(const chunk &arg)
    const
{
    return (llo == arg.llo && rlo == arg.rlo);
}


bool
chunk::operator<(const chunk &arg)
    const
{
    return (llo != arg.llo ? llo < arg.llo : rlo < arg.rlo);
}


rcstring
chunk::representation()
    const
{
    return
        rcstring::printf
        (
            "{%ld..%ld,%ld..%ld}",
            (long)llo,
            (long)lhi,
            (long)rlo,
            (long)rhi
        );
}


void
chunk::print()
    const
{
    printf("%s\n", representation().c_str());
}
