//
//      contrast - compare files token by token
//      Copyright (C) 2006, 2009 Peter Miller
//
//      This program is free software; you can redistribute it and/or
//      modify it under the terms of the GNU General Public License,
//      version 3, as published by the Free Software Foundation.
//
//      This program is distributed in the hope that it will be useful,
//      but WITHOUT ANY WARRANTY; without even the implied warranty of
//      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//      GNU General Public License for more details.
//
//      You should have received a copy of the GNU General Public License
//      along with this program. If not, see
//      <http://www.gnu.org/licenses/>.
//

#ifndef LIB_CHUNK_H
#define LIB_CHUNK_H

#include <cstddef>

class rcstring; // forward

/**
  * The chunk class is used to represent a segment of two files which
  * are either completely alike or completely unalike.
  */
class chunk
{
public:
    /**
      * The destructor.
      */
    virtual ~chunk();

    /**
      * The default constructor.
      */
    chunk();

    /**
      * The constructor.
      */
    chunk(size_t llo, size_t lhi, size_t rlo, size_t rhi);

    /**
      * The copy constructor.
      */
    chunk(const chunk &);

    /**
      * The assignment operator.
      */
    chunk &operator=(const chunk &);

    bool operator==(const chunk &rhs) const;
    bool operator!=(const chunk &rhs) const;
    bool operator<(const chunk &rhs) const;
    bool operator<=(const chunk &rhs) const;
    bool operator>(const chunk &rhs) const;
    bool operator>=(const chunk &rhs) const;

    bool valid() const;
    bool empty() const { return (llo == lhi && rlo == rhi); }

    size_t get_llo() const { return llo; }
    size_t get_lhi() const { return lhi; }
    size_t get_rlo() const { return rlo; }
    size_t get_rhi() const { return rhi; }

    size_t get_left_width() const { return (lhi - llo); }
    size_t get_right_width() const { return (rhi - rlo); }

    rcstring representation() const;
    void print() const;

    bool
    is_unchanged()
        const
    {
        return (get_left_width() != 0 && get_right_width() != 0);
    }

    bool
    is_delete()
        const
    {
        return (get_left_width() != 0 && get_right_width() == 0);
    }

    bool
    is_insert()
        const
    {
        return (get_left_width() == 0 && get_right_width() != 0);
    }

private:
    size_t llo;
    size_t lhi;
    size_t rlo;
    size_t rhi;
};

#endif // LIB_CHUNK_H
