//
//      contrast - compare files token by token
//      Copyright (C) 2006, 2009 Peter Miller
//
//      This program is free software; you can redistribute it and/or
//      modify it under the terms of the GNU General Public License,
//      version 3, as published by the Free Software Foundation.
//
//      This program is distributed in the hope that it will be useful,
//      but WITHOUT ANY WARRANTY; without even the implied warranty of
//      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//      GNU General Public License for more details.
//
//      You should have received a copy of the GNU General Public License
//      along with this program. If not, see
//      <http://www.gnu.org/licenses/>.
//

#ifndef LIB_CHUNK_LIST_H
#define LIB_CHUNK_LIST_H

#include <lib/chunk.h>

/**
  * The chunk_list class is used to represent an ordered sequence of
  * chunks.
  */
class chunk_list
{
public:
    /**
      * The destructor.
      */
    virtual ~chunk_list();

    /**
      * The default constructor.
      */
    chunk_list();

    /**
      * The copy constructor.
      */
    chunk_list(const chunk_list &);

    /**
      * The assignment operator.
      */
    chunk_list &operator=(const chunk_list &);

    void push_back(const chunk &arg);

    void sort();

    void print() const;

    bool empty() const { return (length == 0); }
    size_t size() const { return length; }
    const chunk &get(size_t n) const { return item[n]; }
    const chunk &operator[](size_t n) const { return get(n); }

private:
    chunk *item;
    size_t length;
    size_t maximum;
};

#endif // LIB_CHUNK_LIST_H
