//
//      contrast - compare files token by token
//      Copyright (C) 2006, 2009 Peter Miller
//
//      This program is free software; you can redistribute it and/or
//      modify it under the terms of the GNU General Public License,
//      version 3, as published by the Free Software Foundation.
//
//      This program is distributed in the hope that it will be useful,
//      but WITHOUT ANY WARRANTY; without even the implied warranty of
//      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//      GNU General Public License for more details.
//
//      You should have received a copy of the GNU General Public License
//      along with this program. If not, see
//      <http://www.gnu.org/licenses/>.
//

#ifndef LIB_COMPARATOR_MYERS_EGGERT_H
#define LIB_COMPARATOR_MYERS_EGGERT_H

#include <lib/comparator.h>

/**
  * The comparator_myers_eggert class is used to represent
  */
class comparator_myers_eggert:
    public comparator
{
public:
    /**
      * The destructor.
      */
    virtual ~comparator_myers_eggert();

    /**
      * The constructor.
      */
    comparator_myers_eggert(const token_list &lhs, const token_list &rhs);

protected:
    bool find_equal_region(const chunk &region, chunk &equal_region);

private:
    long *xvec;
    long *yvec; // Vectors being compared.

    long *fdiag_base;

    /**
      * The fdiag instance variable is used to remember a vector,
      * indexed by diagonal, containing 1 + the X coordinate of the
      * point furthest along the given diagonal in the forward search of
      * the edit matrix.
      */
    long *fdiag;

    long *bdiag_base;

    /**
      * The bdiag instance variable is used to remember a vector,
      * indexed by diagonal, containing the X coordinate of the point
      * furthest along the given diagonal in the backward search of the
      * edit matrix.
      */
    long *bdiag;

    /**
      * The too_expensive instance variable is used to remember the
      * Edit scripts longer than this are too expensive to compute.
      */
    long too_expensive;

    enum
    {
        /**
          * Snakes bigger than this are considered `big'.
          */
        SNAKE_LIMIT = 20
    };

    /**
      * The default constructor.  Do not use.
      */
    comparator_myers_eggert();

    /**
      * The copy constructor.  Do not use.
      */
    comparator_myers_eggert(const comparator_myers_eggert &);

    /**
      * The assignment operator.  Do not use.
      */
    comparator_myers_eggert &operator=(const comparator_myers_eggert &);
};

#endif // LIB_COMPARATOR_MYERS_EGGERT_H
