//
//      contrast - compare files token by token
//      Copyright (C) 2006, 2009 Peter Miller
//
//      This program is free software; you can redistribute it and/or
//      modify it under the terms of the GNU General Public License,
//      version 3, as published by the Free Software Foundation.
//
//      This program is distributed in the hope that it will be useful,
//      but WITHOUT ANY WARRANTY; without even the implied warranty of
//      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//      GNU General Public License for more details.
//
//      You should have received a copy of the GNU General Public License
//      along with this program. If not, see
//      <http://www.gnu.org/licenses/>.
//

#ifndef LIB_COMPARATOR_MYERS_H
#define LIB_COMPARATOR_MYERS_H

#include <lib/comparator.h>


/**
  * The comparator_myers class is used to represent the Myers O(ND)
  * difference algorithm method of computing the difference between
  * strings.
  *
  * Reference:
  *     "An O(ND) Difference Algorithm and Its Variations", Eugene
  *     W. Myers, TR 85-6, 10-April-1985, Department of Computer
  *     Science, The University of Arizona, Tuscon, Arizona 85721.
  */
class comparator_myers:
    public comparator
{
public:
    /**
      * The destructor.
      */
    virtual ~comparator_myers();

    /**
      * The constructor.
      *
      * @param lhs
      *     The left hand side of the comparison.  The value referenced
      *     MUST persist for longer than the life time of this object.
      * @param rhs
      *     The right hand side of the comparison.  The value referenced
      *     MUST persist for longer than the life time of this object.
      */
    comparator_myers(const token_list &lhs, const token_list &rhs);

protected:
    // See base class for documentation.
    bool find_equal_region(const chunk &region, chunk &equal_region);

private:
    long *v1_table;
    long *v2_table;
    long *v1;
    long *v2;

    /**
      * The default constructor.  Do not use.
      */
    comparator_myers();

    /**
      * The copy constructor.  Do not use.
      */
    comparator_myers(const comparator_myers &);

    /**
      * The assignment operator.  Do not use.
      */
    comparator_myers &operator=(const comparator_myers &);
};

#endif // LIB_COMPARATOR_MYERS_H
