//
//      example-c-plus-plus - an example Aegis project
//      Copyright (C) 2006, 2009, 2010 Peter Miller
//
//      This program is free software; you can redistribute it and/or
//      modify it under the terms of the GNU General Public License,
//      version 3, as published by the Free Software Foundation.
//
//      This program is distributed in the hope that it will be useful,
//      but WITHOUT ANY WARRANTY; without even the implied warranty of
//      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//      GNU General Public License for more details.
//
//      You should have received a copy of the GNU General Public License
//      along with this program. If not, see
//      <http://www.gnu.org/licenses/>.
//

#include <cstring>

#include <lib/rcstring.h>


rcstring
rcstring::basename(const rcstring &suffix)
    const
{
    if (empty())
        return ".";

    const char *begin = c_str();
    const char *end = begin + size();

    //
    // Ignore trailing '/' characters.
    //
    while (end > begin + 1 && end[-1] == '/')
        --end;

    //
    // Ignore suffix
    //
    if
    (
        !suffix.empty()
    &&
        begin + suffix.size() <= end
    &&
        0 == memcmp(end - suffix.size(), suffix.c_str(), suffix.size())
    )
    {
        end -= suffix.size();
    }

    //
    // If all that is left is a single slash,
    // the return root ("/").
    //
    if (end - begin == 1 && *begin == '/')
        return "/";

    //
    // Now we have to find the last slash.
    //
    const char *cp = end;
    while (cp > begin)
    {
        //
        // If path does not contain a slash,
        // dirname returns the original path.
        //
        if (cp == begin)
        {
            return rcstring(begin, end - begin);
        }
        if (cp[-1] == '/')
            break;
        --cp;
    }

    //
    // Build the result.
    //
    return rcstring(cp, end - cp);
}
