//
//      contrast - compare files token by token
//      Copyright (C) 2006, 2009, 2010 Peter Miller
//
//      This program is free software; you can redistribute it and/or
//      modify it under the terms of the GNU General Public License,
//      version 3, as published by the Free Software Foundation.
//
//      This program is distributed in the hope that it will be useful,
//      but WITHOUT ANY WARRANTY; without even the implied warranty of
//      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//      GNU General Public License for more details.
//
//      You should have received a copy of the GNU General Public License
//      along with this program. If not, see
//      <http://www.gnu.org/licenses/>.
//


#include <lib/config.h>
#include <cassert>
#include <libexplain/close.h>
#include <libexplain/open.h>
#include <libexplain/read.h>

#include <fcntl.h>
#include <unistd.h>

#include <lib/source.h>


source::~source()
{
    if (fd >= 0)
    {
        explain_close_or_die(fd);
        fd = -1;
    }
    if (buffer)
    {
        delete [] buffer;
        buffer = 0;
        buffer_size = 0;
        buffer_pos = 0;
        buffer_end = 0;
    }
}


source::source(const rcstring &a_filename) :
    filename(a_filename),
    line_number(1),
    fd(-1),
    buffer(0),
    buffer_size(0),
    buffer_pos(0),
    buffer_end(0),
    pushback_length(0)
{
}


int
source::underflow()
{
    if (pushback_length > 0)
        return pushback[--pushback_length];
    if (fd < 0)
    {
        fd = explain_open_or_die(filename.c_str(), O_RDONLY, 0);
        assert(fd >= 0);
        buffer_size = 1 << 14;
        buffer = new unsigned char [buffer_size];
        buffer_pos = buffer;
        buffer_end = buffer;
    }
    if (buffer_pos >= buffer_end)
    {
        ssize_t n = explain_read_or_die(fd, buffer, buffer_size);
        if (n == 0)
            return -1;
        buffer_pos = buffer;
        buffer_end = buffer + n;
    }
    return *buffer_pos++;
}


void
source::ungetch(int c)
{
    if (c < 0)
        return;
    assert(pushback_length < sizeof(pushback));
    pushback[pushback_length++] = c;
}
