//
//      contrast - compare files token by token
//      Copyright (C) 2006, 2009 Peter Miller
//
//      This program is free software; you can redistribute it and/or
//      modify it under the terms of the GNU General Public License,
//      version 3, as published by the Free Software Foundation.
//
//      This program is distributed in the hope that it will be useful,
//      but WITHOUT ANY WARRANTY; without even the implied warranty of
//      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//      GNU General Public License for more details.
//
//      You should have received a copy of the GNU General Public License
//      along with this program. If not, see
//      <http://www.gnu.org/licenses/>.
//

#ifndef LIB_SOURCE_C_H
#define LIB_SOURCE_C_H

#include <lib/rcstring/accumulator.h>
#include <lib/source.h>

/**
  * The source_c class is used to represent a token stream from a C (or
  * similar) programming language source file.
  */
class source_c:
    public source
{
public:
    /**
      * The destructor.
      */
    virtual ~source_c();

    /**
      * The constructor.
      */
    source_c(const rcstring &filename);

protected:
    // See base class for documentation.
    type_t get_next_token(rcstring &value);

private:
    /**
      * The ac instance variale is used to remember the text of the
      * current token as it is being determined.  While this could have
      * been local to get_next_token, it is more efficient if we don't
      * have to malloc all the time.
      */
    rcstring_accumulator ac;

    enum state_t
    {
        state_normal,
        state_c_comment,
        state_cpp_comment,
        state_string_constant,
        state_char_constant
    };

    /**
      * The state instance variable i sused to remember which context the
      * tokenization is ocurring in, because each has different tokens.
      */
    state_t state;

    /**
      * The get_normal_token method is used to obtain the next token
      * from the input source.  This is for everything except comments
      * and strings and character constants.
      */
    type_t get_normal_token(rcstring &value);

    /**
      * The get_c_comment_token method is used to obtain the next token
      * from the input source.  This is for breaking up C comments into
      * words, much as the generic tokenizer does.
      */
    type_t get_c_comment_token(rcstring &value);

    /**
      * The get_cpp_comment_token method is used to obtain the next
      * token from the input source.  This is for breaking up C++
      * comments into words, much as the generic tokenizer does.
      */
    type_t get_cpp_comment_token(rcstring &value);

    /**
      * The get_string_token method is used to obtain the next token
      * from the input source.  This is for strings and character
      * constants.
      */
    type_t get_string_token(rcstring &value, unsigned char terminator);

    /**
      * The default constructor.  Do not use.
      */
    source_c();

    /**
      * The copy constructor.  Do not use.
      */
    source_c(const source_c &);

    /**
      * The assignment operator.  Do not use.
      */
    source_c &operator=(const source_c &);
};

#endif // LIB_SOURCE_C_H
