//
//      contrast - compare files token by token
//      Copyright (C) 2006, 2009 Peter Miller
//
//      This program is free software; you can redistribute it and/or
//      modify it under the terms of the GNU General Public License,
//      version 3, as published by the Free Software Foundation.
//
//      This program is distributed in the hope that it will be useful,
//      but WITHOUT ANY WARRANTY; without even the implied warranty of
//      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//      GNU General Public License for more details.
//
//      You should have received a copy of the GNU General Public License
//      along with this program. If not, see
//      <http://www.gnu.org/licenses/>.
//

#include <cctype>

#include <lib/rcstring/accumulator.h>
#include <lib/source/generic.h>


source_generic::~source_generic()
{
}


source_generic::source_generic(const rcstring &a_filename) :
    source(a_filename)
{
}


static bool
part_of_word(unsigned char c)
{
    switch (c)
    {
    case '$':
    case '.':
    case '0':
    case '1':
    case '2':
    case '3':
    case '4':
    case '5':
    case '6':
    case '7':
    case '8':
    case '9':
    case 'A':
    case 'B':
    case 'C':
    case 'D':
    case 'E':
    case 'F':
    case 'G':
    case 'H':
    case 'I':
    case 'J':
    case 'K':
    case 'L':
    case 'M':
    case 'N':
    case 'O':
    case 'P':
    case 'Q':
    case 'R':
    case 'S':
    case 'T':
    case 'U':
    case 'V':
    case 'W':
    case 'X':
    case 'Y':
    case 'Z':
    case '_':
    case 'a':
    case 'b':
    case 'c':
    case 'd':
    case 'e':
    case 'f':
    case 'g':
    case 'h':
    case 'i':
    case 'j':
    case 'k':
    case 'l':
    case 'm':
    case 'n':
    case 'o':
    case 'p':
    case 'q':
    case 'r':
    case 's':
    case 't':
    case 'u':
    case 'v':
    case 'w':
    case 'x':
    case 'y':
    case 'z':
        return true;
    }
    return false;
}


source::type_t
source_generic::get_next_token(rcstring &value)
{
    static rcstring_accumulator ac;
    ac.clear();
    int c = getch();
    if (c < 0)
    {
        value = "";
        return type_end_of_file;
    }
    unsigned char uc = c;

    //
    // See if it looks like white space.
    //
    if (isspace(uc))
    {
        for (;;)
        {
            ac.push_back(uc);
            c = getch();
            if (c < 0)
                break;
            uc = c;
            if (!isspace(uc))
            {
                ungetch(uc);
                break;
            }
        }
        value = ac.mkstr();
        return type_whitespace;
    }

    //
    // See if it looks like punctuation between words.
    //
    if (!part_of_word(uc))
    {
        for (;;)
        {
            ac.push_back(uc);
            c = getch();
            if (c < 0)
                break;
            uc = c;
            if (isspace(uc) || part_of_word(uc))
            {
                ungetch(uc);
                break;
            }
        }
        value = ac.mkstr();
        return type_ink;
    }

    //
    // Grab a word.
    //
    for (;;)
    {
        ac.push_back(uc);
        c = getch();
        if (c < 0)
            break;
        uc = c;
        if (!part_of_word(uc))
        {
            ungetch(uc);
            break;
        }
    }
    value = ac.mkstr();
    return type_ink;
}
