//
//      contrast - compare files token by token
//      Copyright (C) 2006, 2009 Peter Miller
//
//      This program is free software; you can redistribute it and/or
//      modify it under the terms of the GNU General Public License,
//      version 3, as published by the Free Software Foundation.
//
//      This program is distributed in the hope that it will be useful,
//      but WITHOUT ANY WARRANTY; without even the implied warranty of
//      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//      GNU General Public License for more details.
//
//      You should have received a copy of the GNU General Public License
//      along with this program. If not, see
//      <http://www.gnu.org/licenses/>.
//

#include <cstring>

#include <lib/rcstring/accumulator.h>
#include <lib/source/lines.h>


source_lines::~source_lines()
{
}


source_lines::source_lines(const rcstring &a_filename) :
    source(a_filename)
{
}


static inline bool
end_of_line_char(int c)
{
    return (c == '\r' || c == '\n' || c == '\f');
}


source::type_t
source_lines::get_next_token(rcstring &value)
{
    static rcstring_accumulator ac;
    ac.clear();
    int c = getch();
    if (c < 0)
    {
        value = "";
        return type_end_of_file;
    }
    unsigned char uc = c;
    if (end_of_line_char(uc))
    {
        for (;;)
        {
            ac.push_back(uc);
            c = getch();
            if (c < 0)
                break;
            uc = c;
            if (!end_of_line_char(uc))
            {
                ungetch(uc);
                break;
            }

            // No repeats
            if (memchr(ac.get_data(), uc, ac.size()))
            {
                ungetch(uc);
                break;
            }
        }
        value = ac.mkstr();
        return type_whitespace;
    }
    if (uc == '\n')
    {
        value = "\n";
        return type_whitespace;
    }
    for (;;)
    {
        ac.push_back(uc);
        c = getch();
        if (c < 0)
            break;
        uc = c;
        if (end_of_line_char(uc))
        {
            ungetch(uc);
            break;
        }
    }
    value = ac.mkstr();
    return type_ink;
}
