//
//      contrast - compare files token by token
//      Copyright (C) 2006, 2009 Peter Miller
//
//      This program is free software; you can redistribute it and/or
//      modify it under the terms of the GNU General Public License,
//      version 3, as published by the Free Software Foundation.
//
//      This program is distributed in the hope that it will be useful,
//      but WITHOUT ANY WARRANTY; without even the implied warranty of
//      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//      GNU General Public License for more details.
//
//      You should have received a copy of the GNU General Public License
//      along with this program. If not, see
//      <http://www.gnu.org/licenses/>.
//

#include <lib/source.h>
#include <lib/token/list.h>


void
token_list::read(const rcstring &filename, const rcstring &tokenizer_name)
{
    source *src = source::factory(filename, tokenizer_name);
    read(src);
    delete src;
}


void
token_list::read(source *src)
{
    rcstring white;
    bool seen_white = false;
    for (;;)
    {
        rcstring value;
        source::type_t type = src->get_next_token(value);
        switch (type)
        {
        case source::type_end_of_file:
            if (seen_white)
                push_back(white, "");
            return;

        case source::type_whitespace:
            if (seen_white)
            {
                push_back(white, "");
                seen_white = false;
            }
            white = value;
            seen_white = true;
            break;

        case source::type_ink:
            push_back((seen_white ? white : ""), value);
            seen_white = false;
            break;
        }
    }
}
