#!/bin/sh
#
#       contrast - compare files token by token
#       Copyright (C) 2006, 2009 Peter Miller
#
#       This program is free software; you can redistribute it and/or
#       modify it under the terms of the GNU General Public License,
#       version 3, as published by the Free Software Foundation.
#
#       This program is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#
#       You should have received a copy of the GNU General Public License
#       along with this program. If not, see
#       <http://www.gnu.org/licenses/>.
#
work=${TMPDIR:-/tmp}/$$
PAGER=cat
export PAGER
umask 022
here=`pwd`
if test $? -ne 0 ; then exit 2; fi
SHELL=/bin/sh
export SHELL

bin="$here/${1-.}/bin"

pass()
{
        set +x
        cd $here
        rm -rf $work
        exit 0
}
fail()
{
        set +x
        echo 'FAILED test of the Myers algorithm functionality' 1>&2
        cd $here
        rm -rf $work
        exit 1
}
no_result()
{
        set +x
        echo 'NO RESULT for test of the Myers algorithm functionality' 1>&2
        cd $here
        rm -rf $work
        exit 2
}
trap \"no_result\" 1 2 3 15

mkdir $work $work/lib
if test $? -ne 0 ; then no_result; fi
cd $work
if test $? -ne 0 ; then no_result; fi

#
# test the Myers algorithm functionality
#
cat > lhs << 'fubar'
int main(int ac, char **av) {
return 0;
}
fubar
test $? -eq 0 || no_result

cat > rhs << 'fubar'
int
main(int argc, char **argv)
{
    return 0;
}
fubar
test $? -eq 0 || no_result

cat > ok << 'fubar'
int main(int argc, char **argv) {
return 0;
}
fubar
test $? -eq 0 || no_result

$bin/contrast-reformat lhs rhs > test.out
test $? -eq 0 || fail

diff ok test.out
test $? -eq 0 || fail

#
# Only definite negatives are possible.
# The functionality exercised by this test appears to work,
# no other guarantees are made.
#
pass
