#!/bin/sh
#
# contrast - compare files token by token
# Copyright (C) 2006, 2009 Peter Miller
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License,
# version 3, as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see
# <http://www.gnu.org/licenses/>.
#
work=${TMPDIR:-/tmp}/$$
PAGER=cat
export PAGER
umask 022
here=`pwd`
if test $? -ne 0 ; then exit 2; fi
SHELL=/bin/sh
export SHELL

bin="$here/${1-.}/bin"

pass()
{
        set +x
        cd $here
        rm -rf $work
        exit 0
}
fail()
{
        set +x
        echo 'FAILED test of the C tokenizer' 1>&2
        cd $here
        rm -rf $work
        exit 1
}
no_result()
{
        set +x
        echo 'NO RESULT for test of the C tokenizer' 1>&2
        cd $here
        rm -rf $work
        exit 2
}
trap \"no_result\" 1 2 3 15

mkdir $work $work/lib
test $? -eq 0 || no_result
cd $work
test $? -eq 0 || no_result

#
# test the C tokenizer
#
cat > test.c << 'fubar'
!
!=
"\xFF"
"str\
ing"
"string"
#
##
%
%=
&
&&
&=
'c'
*
*=
+
++
+=
-
--
-=
->
.
...
/
/* this is a "C" coment */
// this is a 'C++' comment
/=
0123
0x42
4.5e9
456
456.7
<
<<
<<=
<=
>
>=
>>
>>=
[
]
^
name
name$with$dollars
name_with_underscores
{
|
|=
||
}
~
fubar
test $? -eq 0 || no_result

cat > test.ok << 'fubar'
{ white = ""; ink = "!"; },
{ white = "\n"; ink = "!="; },
{ white = "\n"; ink = "\""; },
{ white = ""; ink = "\\x"; },
{ white = ""; ink = "FF"; },
{ white = ""; ink = "\""; },
{ white = "\n"; ink = "\""; },
{ white = ""; ink = "str"; },
{ white = ""; ink = "\\\n"; },
{ white = ""; ink = "ing"; },
{ white = ""; ink = "\""; },
{ white = "\n"; ink = "\""; },
{ white = ""; ink = "string"; },
{ white = ""; ink = "\""; },
{ white = "\n"; ink = "#"; },
{ white = "\n"; ink = "##"; },
{ white = "\n"; ink = "%"; },
{ white = "\n"; ink = "%="; },
{ white = "\n"; ink = "&"; },
{ white = "\n"; ink = "&&"; },
{ white = "\n"; ink = "&="; },
{ white = "\n"; ink = "'"; },
{ white = ""; ink = "c"; },
{ white = ""; ink = "'"; },
{ white = "\n"; ink = "*"; },
{ white = "\n"; ink = "*="; },
{ white = "\n"; ink = "+"; },
{ white = "\n"; ink = "++"; },
{ white = "\n"; ink = "+="; },
{ white = "\n"; ink = "-"; },
{ white = "\n"; ink = "--"; },
{ white = "\n"; ink = "-="; },
{ white = "\n"; ink = "->"; },
{ white = "\n"; ink = "."; },
{ white = "\n"; ink = "..."; },
{ white = "\n"; ink = "/"; },
{ white = "\n"; ink = "/*"; },
{ white = " "; ink = "this"; },
{ white = " "; ink = "is"; },
{ white = " "; ink = "a"; },
{ white = " "; ink = "\""; },
{ white = ""; ink = "C"; },
{ white = ""; ink = "\""; },
{ white = " "; ink = "coment"; },
{ white = " "; ink = "*/"; },
{ white = "\n"; ink = "//"; },
{ white = " "; ink = "this"; },
{ white = " "; ink = "is"; },
{ white = " "; ink = "a"; },
{ white = " "; ink = "'"; },
{ white = ""; ink = "C"; },
{ white = ""; ink = "++'"; },
{ white = " "; ink = "comment"; },
{ white = "\n"; ink = "/="; },
{ white = "\n"; ink = "0123"; },
{ white = "\n"; ink = "0x42"; },
{ white = "\n"; ink = "4.5e9"; },
{ white = "\n"; ink = "456"; },
{ white = "\n"; ink = "456.7"; },
{ white = "\n"; ink = "<"; },
{ white = "\n"; ink = "<<"; },
{ white = "\n"; ink = "<<="; },
{ white = "\n"; ink = "<="; },
{ white = "\n"; ink = ">"; },
{ white = "\n"; ink = ">="; },
{ white = "\n"; ink = ">>"; },
{ white = "\n"; ink = ">>="; },
{ white = "\n"; ink = "["; },
{ white = "\n"; ink = "]"; },
{ white = "\n"; ink = "^"; },
{ white = "\n"; ink = "name"; },
{ white = "\n"; ink = "name$with$dollars"; },
{ white = "\n"; ink = "name_with_underscores"; },
{ white = "\n"; ink = "{"; },
{ white = "\n"; ink = "|"; },
{ white = "\n"; ink = "|="; },
{ white = "\n"; ink = "||"; },
{ white = "\n"; ink = "}"; },
{ white = "\n"; ink = "~"; },
{ white = "\n"; ink = ""; },
fubar
test $? -eq 0 || no_result

$bin/contrast-debug -T test.c > test.out
test $? -eq 0 || fail

diff test.ok test.out
test $? -eq 0 || fail

#
# Only definite negatives are possible.
# The functionality exercised by this test appears to work,
# no other guarantees are made.
#
pass
